﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MyCollections;

namespace MyCollectionsTests
{
    [TestClass]
    public class MultivalueDictionarySortedSetTests
    {

        [TestMethod]
        public void CountPropertyTest()
        {
            AbstractMultiValueDictionary<String, int> dict = new MultiValueDictionarySortedSet<string, int>();
            Assert.AreEqual(0, dict.Count);
        }

        [TestMethod]
        public void AddTest()
        {
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>();
            bool added = dict.Add('A', "Anders");
            Assert.IsTrue(added);
            Assert.AreEqual(1, dict.Count);
            added = dict.Add('A', "Anders");
            Assert.IsFalse(added);
            Assert.AreEqual(1, dict.Count);

            added = dict.Add('A', "Abraham");
            Assert.IsTrue(added);
            Assert.AreEqual(1, dict.Count);

            added = dict.Add('J', "John");
            Assert.IsTrue(added);
            Assert.AreEqual(2, dict.Count);

            try
            {
                dict.Add('A', null);
                Assert.Fail();
            }
            catch (ArgumentNullException)
            {
            }
        }

        [TestMethod]
        public void GetTest()
        {
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>();
            dict.Add('A', "Anders");
            dict.Add('A', "Abraham");
            dict.Add('J', "John");
            Assert.IsNull(dict.Get('G'));
            ISet<String> values = dict.Get('A');
            Assert.AreEqual(2, values.Count);
            Assert.IsTrue(values.Contains("Anders"));
            Assert.IsTrue(values.Contains("Abraham"));
        }

        [TestMethod]
        public void RemoveTest()
        {
            // Object initializer, automatically calling Add(...)
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>
            {
                {'A', "Anders"},
                {'A', "Abraham"},
                {'J', "John"}
            };
            Assert.IsFalse(dict.Remove('N', "NotThere"));
            Assert.IsFalse(dict.Remove('A', "Andrea"));
            Assert.IsTrue(dict.Remove('A', "Anders"));
            Assert.AreEqual(2, dict.Count);
            Assert.IsTrue(dict.Remove('A', "Abraham"));
            Assert.AreEqual(1, dict.Count);
        }

        [TestMethod]
        public void KeysTest()
        {
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>
            {
                {'A', "Anders"},
                {'A', "Abraham"},
                {'J', "John"}
            };
            ICollection<char> keys = dict.Keys;
            Assert.AreEqual(2, keys.Count);
            Assert.IsTrue(keys.Contains('A'));
            Assert.IsTrue(keys.Contains('J'));
        }

        [TestMethod]
        public void ValuesTest()
        {
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>();
            dict.Add('A', "Anders");
            dict.Add('A', "Abraham");
            dict.Add('J', "John");
            ICollection<ISet<String>> values = dict.Values;
            Assert.AreEqual(2, values.Count);
        }


        [TestMethod]
        public void TryGetValueTest()
        {
            AbstractMultiValueDictionary<char, String> dict = new MultiValueDictionarySortedSet<char, String>();
            dict.Add('A', "Anders");
            dict.Add('A', "Abraham");
            dict.Add('J', "John");
            ISet<String> values;
            bool found = dict.TryGetValue('A', out values);
            Assert.IsTrue(found);
            Assert.AreEqual(2, values.Count);
            found = dict.TryGetValue('B', out values);
            Assert.IsFalse(found);
            Assert.IsNull(values);
        }

        [TestMethod]
        public void PutAllTest()
        {
            AbstractMultiValueDictionary<String, String> dict = new MultiValueDictionarySortedSet<String, String>();
            ISet<String> keys = new HashSet<string>();
            keys.Add("Some movie");
            keys.Add("Another movie");
            keys.Add("Illegal movie");
            int howMany = dict.AddAll(keys, "192.11.45.2");
            Assert.AreEqual(3, howMany);
            Assert.AreEqual(3, dict.Count);
            ISet<String> values = dict.Get("Some movie");
            Assert.AreEqual(1, values.Count);
            Assert.IsTrue(values.Contains("192.11.45.2"));
        }

        [TestMethod]
        public void RemoveAllTest()
        {
            AbstractMultiValueDictionary<String, String> dict = new MultiValueDictionarySortedSet<String, String>();
            const string myIP = "192.11.45.2";
            const string anotherIP = "192.44.23.2";
            dict.Add("Some movie", myIP);
            dict.Add("Another movie", myIP);
            dict.Add("Some movie", anotherIP);
            int howMany = dict.RemoveAll(myIP);
            Assert.AreEqual(2, howMany);
            Assert.AreEqual(1, dict.Count);
        }

        [TestMethod]
        public void EqualsHashCodeTest()
        {
            AbstractMultiValueDictionary<String, String> dict1 = new MultiValueDictionarySortedSet<String, String>();
            AbstractMultiValueDictionary<String, String> dict2 = new MultiValueDictionarySortedSet<String, String>();
            Assert.IsTrue(dict1.Equals(dict2));
            Assert.AreEqual(dict1, dict2);
            Assert.AreEqual(dict1.GetHashCode(), dict2.GetHashCode());
            dict1.Add("abc", "def");
            Assert.AreNotEqual(dict1, dict2);
            Assert.AreNotEqual(dict1.GetHashCode(), dict2.GetHashCode());
            dict2.Add("abc", "def");
            Assert.AreEqual(dict1, dict2);
            Assert.AreEqual(dict1.GetHashCode(), dict2.GetHashCode());
        }
    }
}
